#include "main.h"
#include "init.h"
#include "highscore.h"

void SDL_init(){
	const SDL_VideoInfo *videoInfo;

	fprintf(stderr,"Initialising SDL video...\n");

	if(SDL_Init(SDL_INIT_VIDEO)<0){
		fprintf(stderr,"Video initialization failed...\n");
		Quit(1);
	}

	videoInfo=SDL_GetVideoInfo();
	if(!videoInfo){
	    	fprintf(stderr,"Video query failed...\n");
	    	Quit(1);
    	}
    	videoFlags  = SDL_OPENGL;          /* Enable OpenGL in SDL */
    	videoFlags |= SDL_GL_DOUBLEBUFFER; /* Enable double buffering */
    	videoFlags |= SDL_HWPALETTE;       /* Store the palette in hardware */
    	videoFlags |= SDL_RESIZABLE;       /* Enable window resizing */

    	if(videoInfo->hw_available)videoFlags|=SDL_HWSURFACE;
    	else{videoFlags|=SDL_SWSURFACE;}

    	if(videoInfo->blit_hw)videoFlags|=SDL_HWACCEL;
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,1);

	fprintf(stderr,"Setting SDL video mode...\n");

    	surface=SDL_SetVideoMode(SCREEN_WIDTH,SCREEN_HEIGHT,SCREEN_BPP,videoFlags);
	SDL_WM_SetCaption("GLAsteroids", "v0.9");

    	if(!surface){
		fprintf(stderr,"Video mode set failed...\n");
		Quit(1);
	}
	
	
	if(SDL_InitSubSystem(SDL_INIT_AUDIO)<0){
		fprintf(stderr,"Could not initialize audio subsystem. Turning sound off...\n");
		audio=0;
	}
	else{audio=1;}


     	if(Mix_OpenAudio(22060,AUDIO_S16SYS,2,512)<0){
  		fprintf(stderr,"Unable to open audio. Turning sound off...\n");
		audio=0;
  	}
	else{audio=1;}
}


int resizeWindow(int width, int height){
	GLfloat ratio;
	fprintf(stderr,"Resizing window...\n");
	if(height==0)height=1;
	ratio = (GLfloat)width/(GLfloat)height;
	glViewport(0,0,(GLsizei)width,(GLsizei)height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f, ratio, 0.1f, 50000.0f);
    	glMatrixMode(GL_MODELVIEW);
    	glLoadIdentity();
    	return(1);
}

int initGL(){
	fprintf(stderr,"Starting OpenGL...\n");
	glEnable(GL_TEXTURE_2D);
  	glShadeModel(GL_SMOOTH);
  	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
  	glClearDepth(1.0f);
  	glEnable(GL_DEPTH_TEST);
  	glEnable(GL_ALPHA_TEST);
  	glAlphaFunc(GL_GEQUAL, 0.5);
  	glDepthFunc(GL_LEQUAL);
  	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	return(1);
}

void create_font(){
	int i,j;
	fontlist=glGenLists(256);
	fprintf(stderr,"Creating font...\n");
	for(j=0;j<6;j++){
	for(i=0;i<16;i++){
		glNewList(31+fontlist+16*j+i,GL_COMPILE);
			glBindTexture(GL_TEXTURE_2D, texture[11]);
			glBegin(GL_QUADS);
				glTexCoord2f((32*i+0)*0.001953,(32*j+1)*0.003906);    
				glVertex2f(0,32);
   		     	glTexCoord2f((32*i+0)*0.001953,(32*j+31)*0.003906);    
				glVertex2f(0,0);
   			   	glTexCoord2f((32*i+32)*0.001953,(32*j+31)*0.003906);    
				glVertex2f(32,0);
       				glTexCoord2f((32*i+32)*0.001953,(32*j+1)*0.003906);    
				glVertex2f(32,32);
			glEnd();
			glTranslated(20, 0, 0);
		glEndList();
	}
	}
}

void create_astd(){
	glNewList(257,GL_COMPILE);
		glBindTexture(GL_TEXTURE_2D, texture[6]);
		glBegin(GL_QUADS);
			glTexCoord2f( 1.00, 0.00 );
			glVertex3f( -1,  1,  -1);
			glTexCoord2f( 0.00, 0.00 );
			glVertex3f(  1,  1,  -1);
			glTexCoord2f( 0.00, 1.00 );
			glVertex3f(  1, -1,  -1);
			glTexCoord2f( 1.00, 1.00 );
			glVertex3f( -1, -1,  -1);
		glEnd();

		glBindTexture(GL_TEXTURE_2D, texture[6]);
    		glBegin(GL_QUADS);
			glTexCoord2f( 1.00, 0.00 );
			glVertex3f( -1,  1,  1);
			glTexCoord2f( 0.00, 0.00 );
			glVertex3f(  1,  1,  1);
			glTexCoord2f( 0.00, 1.00 );
			glVertex3f(  1, -1,  1);
			glTexCoord2f( 1.00, 1.00 );
			glVertex3f( -1, -1,  1);
		glEnd();

		glBindTexture(GL_TEXTURE_2D, texture[6]);
 	   	glBegin(GL_QUADS);
			glTexCoord2f( 1.00, 0.00 );
			glVertex3f(1,1,-1);
			glTexCoord2f( 0.00, 0.00 );
			glVertex3f(1,1,1);
			glTexCoord2f( 0.00, 1.00 );
			glVertex3f(1,-1,1);
			glTexCoord2f( 1.00, 1.00 );
			glVertex3f(1,-1,-1);
		glEnd();

		glBindTexture(GL_TEXTURE_2D, texture[6]);
    		glBegin(GL_QUADS);
			glTexCoord2f( 1.00, 0.00 );
			glVertex3f(-1,  1,  -1);
			glTexCoord2f( 0.00, 0.00 );
			glVertex3f(-1,  1,  1);
			glTexCoord2f( 0.00, 1.00 );
			glVertex3f(-1, -1,  1);
			glTexCoord2f( 1.00, 1.00 );
			glVertex3f(-1, -1, -1);
		glEnd();

		glBindTexture(GL_TEXTURE_2D, texture[6]);
    		glBegin(GL_QUADS);
			glTexCoord2f( 1.00, 0.00 );
			glVertex3f(-1, 1,  1);
			glTexCoord2f( 0.00, 0.00 );
			glVertex3f(1,  1, 1);
			glTexCoord2f( 0.00,1.00 );
			glVertex3f( 1, 1, -1);
			glTexCoord2f( 1.00,1.00 );
			glVertex3f(-1, 1, -1);
		glEnd();

		glBindTexture(GL_TEXTURE_2D, texture[6]);
    		glBegin(GL_QUADS);
			glTexCoord2f( 1.00, 0.00 );
			glVertex3f( -1, -1,  1);
			glTexCoord2f( 0.00, 0.00 );
			glVertex3f(  1, -1,  1);
			glTexCoord2f( 0.00, 1.00 );
			glVertex3f(  1, -1, -1);
			glTexCoord2f( 1.00, 1.00 );
			glVertex3f( -1, -1, -1);
		glEnd();
	glEndList();
}

void create_bull(){	
	glNewList(259,GL_COMPILE);
	glBindTexture(GL_TEXTURE_2D,  texture[7]);
	glBegin(GL_QUADS);
        	glTexCoord2f( 1.00, 0.00 );
		glVertex3f( -1,  1,  0);
		glTexCoord2f( 0.00, 0.00 );
		glVertex3f(  1,  1,  0);
		glTexCoord2f( 0.00, 1.00 );
		glVertex3f(  1, -1,  0);
		glTexCoord2f( 1.00, 1.00 );
		glVertex3f( -1, -1,  0);
	glEnd();
	glEndList();

}

void create_fuel(){	
	GLUquadricObj *pObj = gluNewQuadric();

	glNewList(260,GL_COMPILE);
	glPushMatrix();
    	glBindTexture(GL_TEXTURE_2D, texture[12]);
    		gluQuadricTexture(pObj, 1);
    		gluQuadricDrawStyle(pObj, GLU_FILL);
    	glRotatef(0, 1.0f, 0.0f, 0.0f);
    	glTranslatef(0,0,0 );
    	gluCylinder(pObj, 1, 1, 2, 20, 20);
    	glPopMatrix();
    	gluDeleteQuadric(pObj);
	glEndList();

}

void create_skyb(){
	GLfloat xz=10000.00,y=10000.00;
	GLUquadricObj *pObj = gluNewQuadric();
	
	glNewList(258,GL_COMPILE);
	glBindTexture(GL_TEXTURE_2D, texture[0]);
	glBegin(GL_QUADS);
		glTexCoord2f( 1.00, 0.00 );
  		glVertex3f( -xz, y, xz );
		glTexCoord2f( 1.00, 1.00 );
		glVertex3f(  xz, y, xz );
		glTexCoord2f( 0.00, 1.00 );
		glVertex3f(  xz, y, -xz );
		glTexCoord2f( 0.00, 0.00 );
		glVertex3f( -xz, y, -xz );
    	glEnd();

    	glBindTexture(GL_TEXTURE_2D, texture[1]);
    	glBegin(GL_QUADS);
     		glTexCoord2f( 1.00, 1.00 );
      		glVertex3f( -xz, -y, xz );
        	glTexCoord2f( 1.00, 0.00 );
        	glVertex3f( xz , -y, xz );
        	glTexCoord2f( 0.00, 0.00 );
        	glVertex3f(  xz, -y, -xz );
        	glTexCoord2f( 0.00, 1.00 );
        	glVertex3f( -xz, -y, -xz );

    	glEnd();
    
    	glPushMatrix();
    	glBindTexture(GL_TEXTURE_2D, texture[3]);
    	gluQuadricTexture(pObj, 1);
    	gluQuadricDrawStyle(pObj, GLU_FILL);
    	glRotatef(90, 1.0f, 0.0f, 0.0f);
    	glTranslatef(0,0,-10000 );
    	gluCylinder(pObj, 10000, 10000, 20000, 20, 20);
    	glPopMatrix();
    	gluDeleteQuadric(pObj);
	glEndList();
}

void load_textures(char *file, int i){
	SDL_Surface *text_surface;
	char path[strlen(DATA_DIR)+strlen(file)+2];
	int alpha=GL_RGB;
	sprintf(path,"%s/%s",DATA_DIR,file);
	if((text_surface=IMG_Load(path))){
		glGenTextures(1, &texture[i]);
		glBindTexture(GL_TEXTURE_2D, texture[i]);
		if(text_surface->format->BytesPerPixel==4)alpha=GL_RGBA;
		gluBuild2DMipmaps(GL_TEXTURE_2D,alpha,text_surface->w,
				text_surface->h,alpha,GL_UNSIGNED_BYTE,text_surface->pixels);

		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
         	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
         	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
		SDL_FreeSurface(text_surface);
	}
	else{
		fprintf(stderr,"Unable to open %s/%s for loading\n",DATA_DIR,file);
	}
}

void init_objects(ASTEROID *asteroids, SHIP *ships, BULLET *bullets){
	int i;

	fprintf(stderr,"Initialising variables...\n");

	/*Sets up inital values of all variables*/

	for(i=0;i<MAX_ASTEROIDS;i++){
		asteroids->vx=0;
		asteroids->vy=0;
		asteroids->vz=0;
		asteroids->px=0;
		asteroids->py=0;
		asteroids->pz=0;
		asteroids->xrot=0;
		asteroids->yrot=0;
		asteroids->zrot=0;
		asteroids->vxrot=0;
		asteroids->vyrot=0;
		asteroids->vzrot=0;
		asteroids->state=0;
		asteroids->hits=0;
		asteroids->prox=0;
		asteroids++;
	}	
	asteroids-=MAX_ASTEROIDS;

	for(i=0;i!=3;i++){
		srand((unsigned int)(SDL_GetTicks+i+1));
		asteroids->vx=ts*(GLfloat)((rand())/RAND_MAX-0.5);
		asteroids->vy=ts*(GLfloat)((rand())/RAND_MAX-0.5);
		asteroids->vz=ts*(GLfloat)((rand())/RAND_MAX-0.5);
		asteroids->px=-pow(-1,i%2)*100*((GLfloat)(rand())/RAND_MAX+3);
		asteroids->py=-pow(-1,i%2)*0*((GLfloat)(rand())/RAND_MAX+3);
		asteroids->pz=-pow(-1,i%2)*100*((GLfloat)(rand())/RAND_MAX+3);
		asteroids->vxrot=ts*0.5*((GLfloat)(rand())/RAND_MAX);
		asteroids->vyrot=ts*0.5*((GLfloat)(rand())/RAND_MAX);
		asteroids->vzrot=ts*0.5*((GLfloat)(rand())/RAND_MAX);
		asteroids->state=3;
		asteroids->hits=20;
		asteroids++;
	}
	
	for(i=0;i<1;i++){
		ships->xrot=0.0;
		ships->yrot=0.0;
		ships->vx=0.0;
		ships->vy=0.0;
		ships->vz=0.0;
		ships->px=0.0;
		ships->py=0.0;
		ships->pz=0.0;
		ships->vxz=0.0;
		ships->fuel=100;
		ships->state=1;
	}

	for(i=0;i<128;i++){
		bullets->vx=0.0;
		bullets->vy=0.0;
		bullets->vz=0.0;
		bullets->vxz=0.0;
		bullets->px=0.0;
		bullets->py=0.0;
		bullets->pz=0.0;
		bullets->state=0;
		bullets->range=0;
		bullets++;
	}
	
	a=0;s=0;d=0;w=0;space=0;
 	mb1=0,delay=0;
	kills=0,alert=0;
 	timevar=0,reload=6;
	refuel=1,menu=1,finished=1;
}

void load_audio_and_textures(){

	fprintf(stderr,"Loading textures...\n");

	load_textures("sup.jpg",0);
    	load_textures("sdn1.jpg",1);
	load_textures("pz.jpg",2);
    	load_textures("space1.jpg",3);
	load_textures("astlogo.png",4);
    	load_textures("asttex.jpg",6);
    	load_textures("plastex.png",7);    
    	load_textures("aim.png",9);
    	load_textures("font.png",11);
	load_textures("fuel.jpg",12);

	fprintf(stderr,"Creating and compiling models...\n");

    	create_font();
	create_astd();
	create_bull();
	create_skyb();
	create_fuel();

	fprintf(stderr,"Loading audio...\n");

	plasma=NULL;
	proxalert=NULL;
	explo=NULL;
	plasmaChannel=-1;
	proxalertChannel=-1;
	explChannel=-1;

	if(audio){
		if(!(plasma = Mix_LoadWAV("./sounds/plasma.wav")))printf("Could not find plasma.wav\n");
		if(!(proxalert=Mix_LoadWAV("./sounds/ralert.wav")))printf("Could not find ralert.wav\n");
		if(!(explo=Mix_LoadWAV("./sounds/medexpl.wav")))printf("Could not find medexpl.wav\n");
	}
}


int Quit(int ret){
    	if(proxalert)Mix_FreeChunk(proxalert); 
    	if(plasma)Mix_FreeChunk(plasma);
    	if(explo)Mix_FreeChunk(explo);
    	if(audio)Mix_CloseAudio();
    	SDL_Quit();
    	exit(ret);
}
